@@ -18,7 +18,7 @@
 %                                August 2009                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -1006,6 +1006,7 @@ MagickExport size_t GetNumberColors(const Image *image,FILE *file,
 
   char
     color[MagickPathExtent],
+    count[MagickPathExtent],
     hex[MagickPathExtent],
     tuple[MagickPathExtent];
 
@@ -1051,29 +1052,28 @@ MagickExport size_t GetNumberColors(const Image *image,FILE *file,
   {
     pixel=(*p);
     (void) CopyMagickString(tuple,"(",MagickPathExtent);
-    ConcatenateColorComponent(&pixel,RedPixelChannel,X11Compliance,tuple);
+    ConcatenateColorComponent(&pixel,RedPixelChannel,NoCompliance,tuple);
     (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
-    ConcatenateColorComponent(&pixel,GreenPixelChannel,X11Compliance,tuple);
+    ConcatenateColorComponent(&pixel,GreenPixelChannel,NoCompliance,tuple);
     (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
-    ConcatenateColorComponent(&pixel,BluePixelChannel,X11Compliance,tuple);
+    ConcatenateColorComponent(&pixel,BluePixelChannel,NoCompliance,tuple);
     if (pixel.colorspace == CMYKColorspace)
       {
         (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
-        ConcatenateColorComponent(&pixel,BlackPixelChannel,X11Compliance,
+        ConcatenateColorComponent(&pixel,BlackPixelChannel,NoCompliance,
           tuple);
       }
     if (pixel.alpha_trait != UndefinedPixelTrait)
       {
         (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
-        ConcatenateColorComponent(&pixel,AlphaPixelChannel,X11Compliance,
+        ConcatenateColorComponent(&pixel,AlphaPixelChannel,NoCompliance,
           tuple);
       }
     (void) ConcatenateMagickString(tuple,")",MagickPathExtent);
     (void) QueryColorname(image,&pixel,SVGCompliance,color,exception);
     GetColorTuple(&pixel,MagickTrue,hex);
-    (void) FormatLocaleFile(file,"%10.20g",(double) ((MagickOffsetType)
-      p->count));
-    (void) FormatLocaleFile(file,": %s %s %s\n",tuple,hex,color);
+    (void) sprintf(count,"%g:",(double) ((MagickOffsetType) p->count));
+    (void) FormatLocaleFile(file,"    %s %s %s %s\n",count,tuple,hex,color);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
